@@ -1,10 +1,9 @@
+import logging
 import os
 import queue
-import logging
 from contextlib import closing
 
-from mini_buildd import config, util, events, daemon, sbuild, threads, net, changes
-
+from mini_buildd import changes, config, daemon, events, net, sbuild, threads, util
 
 LOG = logging.getLogger(__name__)
 
@@ -15,14 +14,14 @@ class Uploader(threads.EventThread):
 
         self.failed = {}
         for bres_entry in (f for f in config.ROUTES["builds"].path.ifiles("", ".changes") if changes.Buildresult.match(f.path)):
-            LOG.debug(f"Checking buildresult: {bres_entry.path}")
+            LOG.debug("Checking buildresult: %s", bres_entry.path)
             try:
                 bres = changes.Buildresult(bres_entry.path)
                 if not bres.uploaded():
-                    LOG.warning(f"Found upload-pending buildresult: {bres_entry.path}")
+                    LOG.warning("Found upload-pending buildresult: %s", bres_entry.path)
                     self.queue.put(bres)
             except Exception as e:
-                LOG.error(f"Bogus buildresult (ignoring): {bres_entry.path}: {e}")
+                LOG.error("Bogus buildresult (ignoring): %s: %s", bres_entry.path, e)
 
     def run_event(self, event):
         with self.lock:
@@ -44,14 +43,12 @@ class Build(threads.DeferredThread):
         # Note: If the constructor fails, no buildresult would be uploaded (and packaging would hang). Keep it simple && be sure this does not fail on 'normal' error conditions
         self.breq = breq
         self.uploader_queue = uploader_queue
-        self.bres = self.breq.gen_buildresult()
-        self.sbuild = None
+        self.sbuild = sbuild.SBuild(self.breq, ["--keyid", util.daemon().gnupg.get_first_sec_key()])
         super().__init__(limiter=limiter, name=self.breq.bkey)
 
     def cancel(self, who):
-        if self.is_alive() and self.sbuild is not None:
-            self.bres.cset("Canceled-By", who)
-            self.sbuild.cancel()
+        if self.is_alive():
+            self.sbuild.cancel(who)
 
     def run_deferred(self):
         if self._shutdown is config.SHUTDOWN:
@@ -64,19 +61,17 @@ class Build(threads.DeferredThread):
                 gpg.verify(self.breq.file_path)
 
             # Run sbuild
-            self.sbuild = sbuild.SBuild(self.breq)
-            self.sbuild.run(self.bres)
-            LOG.debug(f"{self}: Sbuild finished: {self.bres.cget('Sbuild-Status')}")
+            self.sbuild.run()
 
-            build_changes_file = self.breq.builds_path.join(self.breq.dfn.changes(arch=self.breq["architecture"]))
-            self.bres.save_to(self.bres.builds_path, changes.Base(build_changes_file).tar() if os.path.exists(build_changes_file) else None)
+            build_changes_file = os.path.join(self.breq.dir_path(), self.breq.dfn.changes(arch=self.breq["architecture"]))
+            self.sbuild.bres.save_to(self.breq.dir_path(), changes.Base(build_changes_file).tar() if os.path.exists(build_changes_file) else None)
         except BaseException as e:
             util.log_exception(LOG, f"Internal-Error: {self}", e)
-            self.bres.cset("Internal-Error", str(util.e2http(e)))
-            self.bres.save_to(self.bres.builds_path)
+            self.sbuild.bres.cset("Internal-Error", str(util.e2http(e)))
+            self.sbuild.bres.save_to(self.breq.dir_path())
 
-        util.daemon().events.log(events.Type.BUILT, self.bres)
-        self.uploader_queue.put(self.bres)
+        util.daemon().events.log(events.Type.BUILT, self.sbuild.bres)
+        self.uploader_queue.put(self.sbuild.bres)
 
 
 class Builder(threads.EventThread):
