@@ -1,17 +1,17 @@
-import os
-import re
-import time
-import shutil
 import contextlib
 import glob
-import urllib.request
 import logging
+import os
+import re
+import shutil
+import time
+import urllib.request
 
 import debian.changelog
-import debian.debian_support
 import debian.deb822
+import debian.debian_support
 
-from mini_buildd import config, util, files, changes, call
+from mini_buildd import call, changes, config, files, util
 
 LOG = logging.getLogger(__name__)
 
@@ -181,23 +181,25 @@ class TemplatePackage(util.TmpDir):
     def dsc(self):
         return glob.glob(os.path.join(self.tmpdir, "*.dsc"))[0]
 
+    @classmethod
+    def call(cls, *args, **kwargs):
+        environment = call.taint_env(
+            {
+                "DEBEMAIL": util.daemon().model.email_address,
+                "DEBFULLNAME": util.daemon().model.mbd_fullname(),
+                "GNUPGHOME": util.daemon().gnupg.home,
+            })
+        return call.Call(*args, **kwargs, env=environment)
+
 
 class KeyringPackage(TemplatePackage):
     def __init__(self):
         super().__init__("archive-keyring")
-        model = util.daemon().model
-        gnupg = util.daemon().gnupg
 
-        self.key_id = gnupg.get_first_sec_key()
-        LOG.debug(f"KeyringPackage using key: '{self.key_id}'")
+        self.key_id = util.daemon().gnupg.get_first_sec_key()
+        LOG.debug("KeyringPackage using key: '%s'", self.key_id)
 
-        self.package_name = f"{model.identity}-archive-keyring"
-        self.environment = call.taint_env(
-            {
-                "DEBEMAIL": model.email_address,
-                "DEBFULLNAME": model.mbd_fullname(),
-                "GNUPGHOME": gnupg.home,
-            })
+        self.package_name = f"{util.daemon().model.identity}-archive-keyring"
 
         # Replace %ID%, %MAINT% and %KEY_ID% in all files in package dir.
         for root, _dirs, _files in os.walk(self.package_dir):
@@ -206,13 +208,13 @@ class KeyringPackage(TemplatePackage):
                 new_file = old_file + ".new"
                 with util.fopen(old_file, "r") as of, util.fopen(new_file, "w") as nf:
                     nf.write(util.subst_placeholders(of.read(),
-                                                     {"ID": model.identity,
+                                                     {"ID": util.daemon().model.identity,
                                                       "KEY_ID": self.key_id,
-                                                      "MAINT": f"{model.mbd_fullname()} <{model.email_address}>"}))
+                                                      "MAINT": f"{util.daemon().model.mbd_fullname()} <{util.daemon().model.email_address}>"}))
                 os.rename(new_file, old_file)
 
         # Export public GnuPG key into the package
-        gnupg.export(os.path.join(self.package_dir, self.package_name + ".gpg"), identity=self.key_id)
+        util.daemon().gnupg.export(os.path.join(self.package_dir, self.package_name + ".gpg"), identity=self.key_id)
 
         # Generate sources.lists
         for r in util.models().Repository.objects.all():
@@ -227,6 +229,10 @@ class KeyringPackage(TemplatePackage):
                             with util.fopen(file_name, "w") as f:
                                 f.write(apt_line.get(type_) + "\n")
 
+        # Generate dput-ng profile
+        with util.fopen(os.path.join(self.package_dir, f"{util.daemon().model.mbd_dput_name()}.json"), "w") as f:
+            f.write(util.json_pretty(util.daemon().model.mbd_get_dputng_profile()))
+
         # Generate SSL certificate
         certs_dir = os.path.join(self.package_dir, "certs")
         os.mkdir(certs_dir)
@@ -235,19 +241,17 @@ class KeyringPackage(TemplatePackage):
                 f.write(util.http_endpoint().get_certificate())
 
         # Generate changelog entry
-        call.Call(["debchange",
+        self.call(["debchange",
                    "--create",
                    "--package", self.package_name,
                    "--newversion", self.package_version,
-                   f"Automatic keyring package for archive '{model.identity}'."],
-                  cwd=self.package_dir,
-                  env=self.environment).check()
+                   f"Automatic keyring package for archive '{util.daemon().model.identity}'."],
+                  cwd=self.package_dir).check()
 
-        call.Call(["dpkg-source",
+        self.call(["dpkg-source",
                    "-b",
                    "package"],
-                  cwd=self.tmpdir,
-                  env=self.environment).check()
+                  cwd=self.tmpdir).check()
 
 
 class TestPackage(TemplatePackage):
@@ -255,15 +259,15 @@ class TestPackage(TemplatePackage):
         super().__init__(template)
         self.package_name = template
 
-        call.Call(["debchange",
+        self.call(["debchange",
                    "--newversion", self.package_version,
                    f"Version update '{self.package_version}'."],
                   cwd=self.package_dir).check()
         if auto_ports:
-            call.Call(["debchange",
+            self.call(["debchange",
                        f"MINI_BUILDD_OPTION: auto-ports={','.join(auto_ports)}"],
                       cwd=self.package_dir).check()
-        call.Call(["dpkg-source", "-b", "package"], cwd=self.tmpdir).check()
+        self.call(["dpkg-source", "-b", "package"], cwd=self.tmpdir).check()
 
 
 _DEFAULT_PORT_OPTIONS = ["lintian-mode=ignore"]
@@ -279,16 +283,13 @@ def _port(dsc_url, source, diststr, version, comments=None, options=None, allow_
 
     with util.tmp_dir(prefix="port-") as tmpdir:
         dfn = files.DebianName(source, version)
-        env = call.taint_env({"DEBEMAIL": util.daemon().model.email_address,
-                              "DEBFULLNAME": util.daemon().model.mbd_fullname(),
-                              "GNUPGHOME": util.daemon().gnupg.home})
 
         # Download DSC via dget.
         dget = ["dget", "--download-only"]
         if allow_unauthenticated:
             dget.append("--allow-unauthenticated")
         try:
-            call.Call(dget + [dsc_url], cwd=tmpdir, env=env).check()
+            TemplatePackage.call(dget + [dsc_url], cwd=tmpdir).check()
         except Exception as e:
             raise util.HTTPBadRequest("Given URL can't be downloaded (maybe retry allowing unauthenticated)") from e
 
@@ -297,40 +298,37 @@ def _port(dsc_url, source, diststr, version, comments=None, options=None, allow_
 
         # Unpack DSC (note: dget does not support -x to a dedcicated dir).
         dst = "debian_source_tree"
-        call.Call(["dpkg-source",
-                   "-x",
-                   os.path.basename(dsc_url),
-                   dst],
-                  cwd=tmpdir,
-                  env=env).check()
+        TemplatePackage.call(["dpkg-source",
+                              "-x",
+                              os.path.basename(dsc_url),
+                              dst],
+                             cwd=tmpdir).check()
 
         dst_path = os.path.join(tmpdir, dst)
 
         # Get version and author from original changelog; use the first block not
         with util.fopen(os.path.join(dst_path, "debian", "changelog"), "r") as cl:
             original_author, original_version = Changelog(cl, max_blocks=100).find_first_not(util.daemon().model.email_address)
-        LOG.debug(f"Port: Found original version/author: {original_version}/{original_author}")
+        LOG.debug("Port: Found original version/author: %s/%s", original_version, original_author)
 
         # Change changelog in DST
-        call.Call(["debchange",
-                   "--newversion", version,
-                   "--force-distribution",
-                   "--force-bad-version",
-                   "--preserve",
-                   "--dist", diststr,
-                   f"Automated port via mini-buildd (no changes). Original DSC's SHA1: {original_dsc_sha1sum}."],
-                  cwd=dst_path,
-                  env=env).check()
+        TemplatePackage.call(["debchange",
+                              "--newversion", version,
+                              "--force-distribution",
+                              "--force-bad-version",
+                              "--preserve",
+                              "--dist", diststr,
+                              f"Automated port via mini-buildd (no changes). Original DSC's SHA1: {original_dsc_sha1sum}."],
+                             cwd=dst_path).check()
 
         for entry in changelog_entries():
-            call.Call(["debchange",
-                       "--append",
-                       entry],
-                      cwd=dst_path,
-                      env=env).check()
+            TemplatePackage.call(["debchange",
+                                  "--append",
+                                  entry],
+                                 cwd=dst_path).check()
 
         # Repack DST
-        call.Call(["dpkg-source", "-b", dst], cwd=tmpdir, env=env).check()
+        TemplatePackage.call(["dpkg-source", "-b", dst], cwd=tmpdir).check()
         dsc = os.path.join(tmpdir, dfn.dsc())
         util.daemon().gnupg.sign(dsc)
 
@@ -338,14 +336,13 @@ def _port(dsc_url, source, diststr, version, comments=None, options=None, allow_
         changes_file_path = os.path.join(tmpdir, dfn.changes("source"))
         with util.fopen(changes_file_path, "w+") as out:
             # Note: dpkg-genchanges has a home-brewed options parser. It does not allow, for example, "-v 1.2.3", only "-v1.2.3", so we need to use *one* sequence item for that.
-            call.Call(["dpkg-genchanges",
-                       "-S",
-                       "-sa",
-                       f"-v{original_version}",
-                       f"-D{util.CField('Originally-Changed-By').fullname}={original_author}"],
-                      cwd=dst_path,
-                      env=env,
-                      stdout=out).check()
+            TemplatePackage.call(["dpkg-genchanges",
+                                  "-S",
+                                  "-sa",
+                                  f"-v{original_version}",
+                                  f"-D{util.CField('Originally-Changed-By').fullname}={original_author}"],
+                                 cwd=dst_path,
+                                 stdout=out).check()
 
         # Sign and upload
         util.daemon().gnupg.sign(changes_file_path)
