@@ -1,10 +1,53 @@
+"""
+.. attention:: **sbuild** (retry-workaround in ``gnupg.py``): Spurious build failures ``can't connect to the agent: IPC connect call failed`` (:debbug:`849551`)
+
+.. attention:: **sbuild**: Fails to build "all" packages with "build archall" flag set to arch "x" in case DSP has >= 1 arch "all" and >=1 arch "y" binary package (:debbug:`706086`)
+
+  This is due to sbuild and in in more detail explained in the Debian bug report.
+
+  A bad one-package workaround would be to change the "build archall" flag to arch "y".
+
+.. attention:: **sbuild** (workaround in ``apt-update`` snippet): ``apt-get update`` "File has unexpected size ... Mirror sync in progress?" failures
+
+  This may happen if by chance there are reprepro updates around the same time ``apt-get update`` runs in the build.
+
+  You may reproduce this quite reliably with the testsuite (it's due to the immediate parent package remove, see ``./devel``):
+
+  1. testsuite: Change ``installed-port`` test to (also) port for running codename (``changelog`` + ``./devel``)
+  2. ./devel updatetestall
+  3. ./devel testsuite-packages installed-port
+
+  Internal sbuild snippet works around this problem by retrying ``apt-get update`` -- but you still see the error lines
+  in the buildlog if that happens.
+
+  Solution could be to lock HTTP delivery on any reprepro change, but for now this seems overkill to me.
+
+.. attention:: **sbuild+apt-cacher-ng** (workaround in ``apt-update`` snippet): ``apt-get update`` stalls on parallel access
+
+  **Phenomena**: A build just always remains ongoing && the buildlog just stalls during ``apt-get update``.
+
+  **Repeat bug**:
+
+  .. literalinclude:: ../../examples/mini-buildd/apt-parallel-bug
+    :caption: /usr/share/doc/mini-buildd/examples/apt-parallel-bug
+
+  **Remedy situation at hand**:
+
+  1. Kill corresponding process ``/usr/lib/apt/methods/gpgv`` or ``/usr/lib/apt/methods/http`` (apt-get update may be retried and build continues)
+  2. Cancel resp. build(s) (then retry)
+
+  **Related 3rd party findings**:
+
+    * https://bugs.launchpad.net/ubuntu/+source/apt-cacher-ng/+bug/1983856
+    * https://bugs.launchpad.net/ubuntu/+source/apt/+bug/2003851
+"""
+import logging
 import os
-import time
 import re
 import subprocess
-import logging
+import time
 
-from mini_buildd import util, config, call, files
+from mini_buildd import call, config, files, util
 
 LOG = logging.getLogger(__name__)
 
@@ -41,23 +84,26 @@ class Blocks(files.Dir):
 
     def usage(self):
         default = self.default.strip()
-        header = (f"{self.name}: <block> ...: Ordered sbuild {self.type} blocks (space-separated)\n\n"
-                  f"Default if not given: {self.default}\n\n"
-                  "Available blocks:\n\n")
+        header = (
+            f"Ordered sbuild {self.type} blocks (space-separated)\n"
+            "\n"
+            f"Default if not given: {self.default}\n\n"
+            "Available blocks:\n\n"
+        )
         ml = max(len(name) for name in self.keys())  # "max length" formatting helper
         return header + "\n".join([f"{name:<{ml}}: [{'default' if name in default else 'extra'}] {block.description}" for name, block in self.items()])
 
-    def validate(self, extra_option):
-        """Validate extra option value from user space (string, space separated), return extra option value as list"""
-        eol = extra_option.split()  # extra option list
+    def validate(self, value):
+        """Validate value from user space (string, space separated), return value as list"""
+        eol = value.split()
         unknown = set(eol) - set(self.keys())
         if unknown:
-            raise util.HTTPBadRequest(f"Unknown sbuild {self.type} block(s): {' '.join(unknown)} (check 'Extra Options' configurations in Distributions)")
+            raise util.HTTPBadRequest(f"Unknown sbuild {self.type} block(s): {' '.join(unknown)}")
         return eol
 
-    def line(self, extra_option):
-        self.validate(extra_option)
-        return f"{self.name}: {extra_option}"
+    def line(self, value):
+        self.validate(value)
+        return f"{self.name}: {value}"
 
 
 class ConfigBlocks(Blocks):
@@ -281,103 +327,15 @@ echo "sun-java6-jre shared/accepted-sun-dlj-v1-1 boolean true" | debconf-set-sel
 
 
 class SBuild():
-    def __init__(self, breq):
-        self.breq = breq
-        self.breq.untar(self.breq.builds_path.full)
-
-        self.config_path = self.breq.builds_path.new_sub([CONFIG_DIR])
-        self.sbuildrc_file_path = self.breq.builds_path.join(".sbuildrc")
-        self.buildlog_file_path = self.breq.builds_path.join(self.breq.dfn.buildlog(self.breq["architecture"]))
-
-        self.call = None
-
-        #
-        # Generate .sbuildrc
-        #
-        sbuildrc = files.PerlModule(".sbuildrc", placeholders={"LIBDIR": config.ROUTES["libdir"].path.new_sub([str(self.breq.dist.codename), self.breq["architecture"]], create=True).full})
-
-        # From distribution config
-        CONFIG_BLOCKS.configure(sbuildrc, self.breq.cget(CONFIG_BLOCKS.name, CONFIG_BLOCKS.default))
-
-        # Custom script from distribution config
-        sbuildrc.add_file(self.config_path.join(CONFIG_SBUILDRC_SNIPPET), description="Custom sbuild config (from Distribution)")
-
-        sbuildrc.save_as(self.sbuildrc_file_path)
-
-        #
-        # Generate setup.d/
-        #
-        setup_d = files.Dir(self.breq.builds_path.join(SETUP_D_DIR), enumerate_file_names=0)
-
-        # From distribution config
-        SETUP_BLOCKS.configure(setup_d, self.breq.cget(SETUP_BLOCKS.name, SETUP_BLOCKS.default))
-
-        # Custom script from distribution config
-        custom_buildrequest = self.config_path.join(CONFIG_CHROOT_SETUP_SCRIPT)
-        if os.path.getsize(custom_buildrequest) > 0:
-            setup_d.add(files.AutoScript(custom_buildrequest))
-
-        setup_d.save()
-
-        #
-        # Generate command line
-        #
-        self.cmdline = ["sbuild",
-                        "--dist", self.breq["distribution"],
-                        "--arch", self.breq["architecture"],
-                        "--chroot", self.breq.schroot_name()]
-
-        for name in setup_d.keys():
-            script = os.path.join(setup_d.path, name)
-            self.cmdline += ["--chroot-setup-command", script]
-
-        self.cmdline += ["--build-dep-resolver", self.breq.cget("Build-Dep-Resolver"),
-                         "--keyid", util.daemon().gnupg.get_first_sec_key(),
-                         "--nolog",
-                         "--log-external-command-output",
-                         "--log-external-command-error",
-                         "--no-apt-update"]
-
-        if self.breq.cget("Arch-All"):
-            self.cmdline += ["--arch-all"]
-        else:
-            self.cmdline += ["--no-arch-all"]  # Be sure to set explicitly: sbuild >= 0.77 does not seem to use this as default any more?
-
-        for dependency in self.breq.cget("Add-Depends", "").split(","):
-            self.cmdline += ["--add-depends", dependency]
-
-        lintian_check = self.breq.check_mode("lintian")
-        if lintian_check.mode.value > 0:
-            self.cmdline += ["--run-lintian"]
-            self.cmdline += ["--lintian-opts", lintian_check.lintian_options(self.breq.dist.codename)]
-            self.cmdline += ["--lintian-opts", self.breq.check_extra_options("lintian")]
-        else:
-            self.cmdline += ["--no-run-lintian"]
-
-        piuparts_check = self.breq.check_mode("piuparts")
-        if piuparts_check.mode.value > 0:
-            self.cmdline += ["--run-piuparts"]
-            self.cmdline += ["--piuparts-opts", f"--schroot {self.breq.schroot_name()} --dpkg-force-confdef --warn-on-debsums-errors --warn-on-leftovers-after-purge --warn-on-others --keep-sources-list"]
-            self.cmdline += ["--piuparts-opts", self.breq.check_extra_options("piuparts")]
-        else:
-            self.cmdline += ["--no-run-piuparts"]
-
-        autopkgtest_check = self.breq.check_mode("autopkgtest")
-        if autopkgtest_check.mode.value > 0:
-            self.cmdline += ["--run-autopkgtest"]
-            self.cmdline += ["--autopkgtest-opts", f"-- schroot {self.breq.schroot_name()}"]
-            self.cmdline += ["--autopkgtest-root-args", ""]
-        else:
-            self.cmdline += ["--no-run-autopkgtest"]
-
-        if "sbuild" in config.DEBUG:
-            self.cmdline += ["--debug"]
-
-        self.cmdline += [self.breq.dfn.dsc()]
-
     BUILDLOG_STATUS_REGEX = re.compile("^(Status|Lintian|Piuparts|Autopkgtest|Build-Time|Package-Time|Space|Build-Space): [^ ]+.*$")
 
-    def run(self, bres):
+    def __init__(self, breq, extra_args):
+        self.breq = breq
+        self.extra_args = extra_args
+        self.bres = self.breq.gen_buildresult()
+        self.call = None
+
+    def run(self):
         """
         Run sbuild && update buildresult from build log
 
@@ -389,35 +347,137 @@ class SBuild():
                   though, would be if sbuild would eventually provide a
                   better way to get these values.
         """
+        self.breq.untar(self.breq.dir_path())
+
+        config_path = os.path.join(self.breq.dir_path(), CONFIG_DIR)
+        sbuildrc_file_path = os.path.join(self.breq.dir_path(), ".sbuildrc")
+        buildlog_file_path = os.path.join(self.breq.dir_path(), self.breq.dfn.buildlog(self.breq["architecture"]))
+
+        #
+        # Generate .sbuildrc
+        #
+        sbuildrc = files.PerlModule(".sbuildrc", placeholders={"LIBDIR": config.ROUTES["libdir"].path.new_sub([str(self.breq.dist.codename), self.breq["architecture"]], create=True).full})
+
+        # From distribution config
+        CONFIG_BLOCKS.configure(sbuildrc, self.breq.cget(CONFIG_BLOCKS.name, CONFIG_BLOCKS.default))
+
+        # Custom script from distribution config
+        sbuildrc.add_file(os.path.join(config_path, CONFIG_SBUILDRC_SNIPPET), description="Custom sbuild config (from Distribution)")
+
+        sbuildrc.save_as(sbuildrc_file_path)
+
+        #
+        # Generate setup.d/
+        #
+        setup_d = files.Dir(os.path.join(self.breq.dir_path(), SETUP_D_DIR), enumerate_file_names=0)
+
+        # From distribution config
+        SETUP_BLOCKS.configure(setup_d, self.breq.cget(SETUP_BLOCKS.name, SETUP_BLOCKS.default))
+
+        # Custom script from distribution config
+        custom_buildrequest = os.path.join(config_path, CONFIG_CHROOT_SETUP_SCRIPT)
+        if os.path.getsize(custom_buildrequest) > 0:
+            setup_d.add(files.AutoScript(custom_buildrequest))
+
+        setup_d.save()
+
+        #
+        # Generate command line
+        #
+        cmdline = [
+            "sbuild",
+            "--dist", self.breq["distribution"],
+            "--arch", self.breq["architecture"],
+            "--chroot", self.breq.schroot_name(),
+            "--build-dep-resolver", self.breq.cget("Build-Dep-Resolver"),
+            "--nolog",
+            "--log-external-command-output",
+            "--log-external-command-error",
+            "--no-apt-update",
+        ]
+
+        for name in setup_d.keys():
+            script = os.path.join(setup_d.path, name)
+            cmdline += ["--chroot-setup-command", script]
+
+        if self.breq.cget("Arch-All"):
+            cmdline += ["--arch-all"]
+        else:
+            cmdline += ["--no-arch-all"]  # Be sure to set explicitly: sbuild >= 0.77 does not seem to use this as default any more?
+
+        for dependency in self.breq.cget("Add-Depends", "").split(","):
+            cmdline += ["--add-depends", dependency]
+
+        lintian_check = self.breq.check_mode("lintian")
+        if lintian_check.mode.value > 0:
+            cmdline += ["--run-lintian"]
+            cmdline += ["--lintian-opts", lintian_check.lintian_options(self.breq.dist.codename)]
+            cmdline += ["--lintian-opts", self.breq.check_extra_options("lintian")]
+        else:
+            cmdline += ["--no-run-lintian"]
+
+        piuparts_check = self.breq.check_mode("piuparts")
+        if piuparts_check.mode.value > 0:
+            cmdline += ["--run-piuparts"]
+            cmdline += ["--piuparts-opts", f"--schroot {self.breq.schroot_name()} --dpkg-force-confdef --warn-on-debsums-errors --warn-on-leftovers-after-purge --warn-on-others --keep-sources-list"]
+            cmdline += ["--piuparts-opts", self.breq.check_extra_options("piuparts")]
+        else:
+            cmdline += ["--no-run-piuparts"]
+
+        autopkgtest_check = self.breq.check_mode("autopkgtest")
+        if autopkgtest_check.mode.value > 0:
+            cmdline += ["--run-autopkgtest"]
+            cmdline += ["--autopkgtest-opts", f"-- schroot {self.breq.schroot_name()}"]
+            cmdline += ["--autopkgtest-root-args", ""]
+        else:
+            cmdline += ["--no-run-autopkgtest"]
+
+        if "sbuild" in config.DEBUG:
+            cmdline += ["--debug"]
+
+        cmdline += self.extra_args
+
+        cmdline += [self.breq.dfn.dsc()]
+
+        # Generate Environment
+        env = {
+            "HOME": self.breq.dir_path(),
+            "GNUPGHOME": config.ROUTES["home"].path.join(".gnupg"),
+            "DEB_BUILD_OPTIONS": self.breq.cget("Deb-Build-Options", ""),
+            "DEB_BUILD_PROFILES": self.breq.cget("Deb-Build-Profiles", ""),
+        }
+
         # Run sbuild
-        with util.fopen(self.buildlog_file_path, "w+") as buildlog, subprocess.Popen(
-                self.cmdline,
-                cwd=self.breq.builds_path.full,
-                env=call.taint_env({"HOME": self.breq.builds_path.full,
-                                    "GNUPGHOME": config.ROUTES["home"].path.join(".gnupg"),
-                                    "DEB_BUILD_OPTIONS": self.breq.cget("Deb-Build-Options", ""),
-                                    "DEB_BUILD_PROFILES": self.breq.cget("Deb-Build-Profiles", "")}),
+        with util.fopen(buildlog_file_path, "w+") as buildlog, subprocess.Popen(
+                cmdline,
+                cwd=self.breq.dir_path(),
+                env=call.taint_env(env),
                 stdout=buildlog,
                 stderr=subprocess.STDOUT) as self.call:
-            LOG.info(f"Building {self.breq} (sbuild process '{self.call.pid}')...")
+            LOG.info("%s: Running sbuild from %s (pid=%s)", self.breq, self.breq.dir_path(), self.call.pid)
             retval = self.call.wait()
 
         # Update bres from buildlog
-        bres.add_file(self.buildlog_file_path)
-        with util.fopen(self.buildlog_file_path, errors="replace") as f:
+        self.bres.add_file(buildlog_file_path)
+        with util.fopen(buildlog_file_path, errors="replace") as f:
             for line in f:
                 if self.BUILDLOG_STATUS_REGEX.match(line):
-                    LOG.debug(f"Build log line detected as build status: {line.strip()}")
+                    LOG.debug("Build log line detected as build status: %s", line.strip())
                     s = line.split(":")
-                    bres.cset("Sbuild-" + s[0], s[1].strip())
+                    self.bres.cset("Sbuild-" + s[0], s[1].strip())
 
         # Guarantee to set 'error' sbuild status if retval != 0 and 'Sbuild-Status' could not be parsed from buildlog
-        if retval != 0 and not bres.cget("Sbuild-Status"):
-            bres.cset("Sbuild-Status", "error")
+        if retval != 0 and not self.bres.cget("Sbuild-Status"):
+            self.bres.cset("Sbuild-Status", "error")
 
-    def cancel(self):
+        LOG.debug("%s: Sbuild finished: %s", self.breq, self.bres.cget('Sbuild-Status'))
+
+        return self.bres
+
+    def cancel(self, who):
         if self.call is not None:
-            LOG.info(f"Terminating {self.breq} (sbuild process '{self.call.pid}')...")
+            LOG.info("Terminating %s (sbuild process '%s')...", self.breq, self.call.pid)
+            self.bres.cset("Canceled-By", who)
             self.call.terminate()
 
             # Wait some time (max 6 seconds) to see if terminate() succeeds
@@ -427,5 +487,5 @@ class SBuild():
                     return
 
             # I guess we should kill, then
-            LOG.warning(f"Killing {self.breq} (sbuild process '{self.call.pid}')...")
+            LOG.warning("Killing %s (sbuild process '%s')...", self.breq, self.call.pid)
             self.call.kill()
